// https://openbook.rheinwerk-verlag.de/c_von_a_bis_z/012_c_zeiger_006.htm 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Fehler: Funktion gibt die Adresse
 * einer lokalen Variablen zurück. */
char *test1(void){
   char buffer[10];
   strcpy(buffer, "testwert");
   return buffer;
}

/* Möglichkeit1: Globale Variable */
char buffer1[10];
char *test2(void){
   strcpy(buffer1, "testwert");
   return buffer1;
}

/* Möglichkeit2: Statische Variable */
char *test3(void){
   static char buffer[10];
   strcpy(buffer, "testwert");
   return buffer;
}

/* Möglichkeit3: Speicher vom Heap verwenden, siehe Z. 55, 58 */
char *test4(char *ptr){
   strcpy(ptr, "testwert");
   return ptr;
}

/* Möglichkeit4: Einen Zeiger als Argument übergeben */
char *test5(char *ptr){
   char buffer[10];
   ptr = buffer;
   strcpy(buffer, "testwert");
   return ptr;
}

int main(int argc, char **argv){
   char *ptr;

   ptr = test1();
   printf("test1: %s\n", ptr); // meistens Datenmüll
   
   ptr = test2();
   printf("test2: %s\n", ptr);
   
   ptr = test3();
   printf("test3: %s\n", ptr);
   
   char *buffer = (char *) malloc(10);
   test4(buffer);
   printf("test4: %s\n", ptr);
   free(buffer);
   
   test5(ptr);
   printf("test5: %s\n", ptr);

   return EXIT_SUCCESS;
}

