#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <errno.h>


/**
 * This function is expected to be executed by a child process. It 
 * expects the child number (e.g. 1 for the first child, 2 for the second child ...)
 * as argument.
 */ 
void print_random_numbers(int child_number)
{
	int r = rand();
	printf("Child number %d: My PID is %d - my first random number is: %d\n", child_number, getpid(), r);
	r = rand();
	printf("Child number %d: My PID is %d - my second random number is: %d\n", child_number, getpid(), r);
	r = rand();
	printf("Child number %d: My PID is %d - my third random number is: %d\n", child_number, getpid(), r);
}

/**
 * This function is expected to be executed by the parent process. It
 * waits for all children to terminate and then returns
 */
void wait_for_children()
{
	pid_t pid;
	int waitstatus; 
	while (-1 != (pid = wait(&waitstatus)) && errno != ECHILD) {
		printf("Parent: Child with pid %d terminated\n", pid);
	}
	printf("Parent: All children terminated!\n");
}

int main(int argc, char** argv)
{
	srand(time(NULL));

	if (argc != 2){
		fprintf(stderr, "Usage: %s <number_of_children>\n", argv[0]);
		exit(-1);
	}
	int children = atoi(argv[1]);
		
	printf("Parent: Starting %d child process to calculate many different random numbers!\n", children);
	for (int i = 1; i <= children; ++i) {
		printf("Parent: Starting child number %d\n", i);
		pid_t p = fork();
		if (p == 0) {
			// child 
			print_random_numbers(i);
			exit(0);
		} else if (p > 0) {
			// parent
			printf("Parent: Started child number %d with PID %d\n", i, p);
		} else {
			//error
			perror("Could not fork");
			exit(1);
		}
	}
	wait_for_children();

	printf("\n\nFinished. Press Enter to quit the program.");
	getchar();
}
