
/**
 * Vergleichen Sie fork3.c und threads2.c Beispiele.
 * 
 * In beiden wird ein Aray ints[] erstellt. 
 * Mit Hilfe von mehreren Prozessen oder Threads werden Teile
 * des Arraus bearbeitet werden. 
 * Vergleichen Sie die Ausfuerungszeit
 * - fuer 40 und 400 Processe/Threads
 * - und fuer 10000 und 5000000 Array Elemente
 * 
 **/

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/time.h>
/* 
 * //Ubuntu:
 * #include <sys/wait.h>
 * #include <time.h>
 */


#define NUM_CHILDS 40
#define NUM_INTS 5000000

int ints[NUM_INTS];

void partial_sum(int i) {
    int start = i * (NUM_INTS / NUM_CHILDS);
    int end = start + (NUM_INTS / NUM_CHILDS);
    int sum = 0;
    
    for (int i = start; i < end; i++) {
        sum += ints[i];
    }
    printf("Zwischensumme = %d aus Process %d\n", sum, i);
}

int main(int argc, char **argv){
	pid_t child_pid;
	srand(time(NULL));
	for (int i = 0; i < NUM_INTS; i++) {
        ints[i] = rand() % 1000 - 500;  //Randomzahl -500 ... +499 
    }

    struct timeval start_time;
    gettimeofday(&start_time, NULL);

    // Kindprozesse erstellen und starten
    for (int i = 0; i < NUM_CHILDS; i++) {
        child_pid = fork();
        if (child_pid == 0) { 	// Kindprozess
            partial_sum(i);		// Summe berechnen
            return 0;			// Process beenden
        } else if (child_pid < 0) { // Fehler beim Forken
            printf("Fehler: Forken fehlgeschlagen.\n");
            return -1;
        }
    }
    
    // Auf Kindprozesse warten
    for (int i = 0; i < NUM_CHILDS; i++) {
        int status;
        pid_t pid = wait(&status);
        if (pid == -1) { // Fehler beim Warten auf Kindprozess
            printf("Fehler: Warten auf Kindprozess fehlgeschlagen.\n");
            return -1;
        }
    }
    struct timeval end_time;
    gettimeofday(&end_time, NULL);
    printf("Ausfuerungszeit: %ld us\n", 
		((end_time.tv_sec - start_time.tv_sec) * 1000000) 
		+ (end_time.tv_usec - start_time.tv_usec));

        
    return 0;
    
}
