/***
 * In diesem Beispiel wird die Funktion thread_func() als Thread-Funktion 
 * verwendet. Diese Funktion erhält einen Integer-Parameter, der 
 * die Nummer des Threads angibt. Die Funktion gibt einfach eine 
 * Start- und eine Endemeldung aus und beendet den Thread. 
 * 
 * Im Hauptprogramm werden drei Threads erstellt und gestartet. 
 * Jeder Thread erhält eine andere Nummer als Parameter. Dann wartet 
 * das Hauptprogramm darauf, dass alle Threads beendet sind, indem es 
 * pthread_join() für jeden Thread aufruft. Schließlich gibt das 
 * Hauptprogramm eine Nachricht aus, dass alle Threads beendet sind.
 * 
 * 
 **/

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>

void *thread_func(void *arg) {
    int thread_num = *(int*)arg;
    printf("Thread %d started.\n", thread_num);
    // Hier könnte man Thread-spezifische Aufgaben ausführen
    printf("Thread %d finished.\n", thread_num);
    pthread_exit(NULL);
}

int main() {
    pthread_t threads[3];
    int thread_args[3];
    int i;

    // Threads erstellen und starten
    for (i = 0; i < 3; i++) {
        thread_args[i] = i;
        int rc = pthread_create(&threads[i], NULL, thread_func,
			(void*)&thread_args[i]);
        if (rc) {
            printf("Fehler: pthread_create() gab %d zurück.\n", rc);
            exit(-1);
        }
    }

    // Auf Threads warten
    for (i = 0; i < 3; i++) {
        int rc = pthread_join(threads[i], NULL);
        if (rc) {
            printf("Fehler: pthread_join() gab %d zurück.\n", rc);
            exit(-1);
        }
    }

    printf("Alle Threads beendet.\n");

    return 0;
}
