/**
 * Vergleichen Sie fork3.c und threads2.c Beispiele.
 * 
 * In beiden wird ein Aray ints[] erstellt. 
 * Mit Hilfe von mehreren Prozessen oder Threads werden Teile
 * des Arraus bearbeitet werden. 
 * Vergleichen Sie die Ausfuerungszeit
 * - fuer 40 und 400 Processe/Threads
 * - und fuer 10000 und 5000000 Array Elemente
 * 
 **/
 
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <sys/time.h>

#define THREADS 40
#define NUM_INTS 5000000

int ints[NUM_INTS];
int sum[THREADS] = {0};

void* thread_sum(void* arg) {
    int thread_id = *(int*) arg;
    int start = thread_id * (NUM_INTS / THREADS);
    int end = start + (NUM_INTS / THREADS);
    int localsum = 0;
    
    for (int i = start; i < end; i++) {
        localsum += ints[i];
    }
    
    sum[thread_id] = localsum;
    
    printf("Zwischensumme =  %d aus Thread %d\n", localsum, thread_id);    
    pthread_exit(NULL);
}

int main() {
    srand(time(NULL));
    
    for (int i = 0; i < NUM_INTS; i++) {
        //ints[i] = rand() % 100;
        ints[i] = rand() % 1000 - 500;  //Randomzahl -500 ... +499
    }
    
    pthread_t threads[THREADS];
    int thread_ids[THREADS];
    
    struct timeval start_time;
    gettimeofday(&start_time, NULL);

    
    for (int i = 0; i < THREADS; i++) {
        thread_ids[i] = i;
        pthread_create(&threads[i], NULL, thread_sum, &thread_ids[i]);
    }
    
    for (int i = 0; i < THREADS; i++) {
        pthread_join(threads[i], NULL);	// Auf Ende von Threads warten
    }
    
    struct timeval end_time;
    gettimeofday(&end_time, NULL);
    printf("Ausfuerungszeit: %ld us\n", 
		((end_time.tv_sec - start_time.tv_sec) * 1000000) 
		+ (end_time.tv_usec - start_time.tv_usec));
    
    int total_sum = 0;
    for (int i = 0; i < THREADS; i++) {
        total_sum += sum[i];
    }
    
    printf("Summe des Arrays: %d\n", total_sum);

    return 0;
}
