// https://www.geeksforgeeks.org/chat-application-between-two-processes-using-signals-and-shared-memory/
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/types.h>
#include <unistd.h>

#define FILLED 0
#define Ready 1
#define NotReady -1

struct memory {
	char buff[100];
	int status, pid1, pid2;
};

struct memory* shmptr;

//shared memory id
int shmid;

// key value of shared memory
int key = 12345;

// handler function to print message received from user2
// if "stop" is received: kill user2 and user1
void handler(int signum){
	// if signum is SIGUSR1, then user 1 is receiving a message from user2
	if (signum == SIGUSR1) {
		printf("Received User2: ");
		puts(shmptr->buff);
		if(strstr(shmptr->buff, "stop") != NULL){ // "stop" found
			int pid1, pid2;
			pid1 = shmptr->pid1;
			pid2 = shmptr->pid2;
			// detaching the shared memory; https://manpages.ubuntu.com/manpages/bionic/de/man2/shmat.2.html
			shmdt((void*)shmptr);	
			// deleting the shared memory; https://linux.die.net/man/2/shmctl
			shmctl(shmid, IPC_RMID, NULL);
			kill(pid2, SIGKILL);	//kill the opposite process
			kill(pid1, SIGKILL);	//kill myself
		}
	}
}

int main(){
	// process id of user1
	int pid = getpid();


	// shared memory create; https://man7.org/linux/man-pages/man2/shmget.2.html
	shmid = shmget(key, sizeof(struct memory), IPC_CREAT | 0666);

	// attaching the shared memory; https://manpages.ubuntu.com/manpages/bionic/de/man2/shmat.2.html
	shmptr = (struct memory*)shmat(shmid, NULL, 0);

	// store the process id of user1 in shared memory
	shmptr->pid1 = pid;
	shmptr->status = NotReady;

	// calling the signal function using signal type SIGUSER1
	signal(SIGUSR1, handler);

	while (1) {
		while (shmptr->status != Ready)
			continue;
			
		sleep(1);

		// taking input from user1
		printf("User1: ");
		fgets(shmptr->buff, 100, stdin);
		shmptr->status = FILLED;

		// sending the message to user2 using kill function
		kill(shmptr->pid2, SIGUSR2);
	}

	//never reached!

	// detaching the shared memory; https://manpages.ubuntu.com/manpages/bionic/de/man2/shmat.2.html
	//shmdt((void*)shmptr);
	
	// deleting the shared memory; https://linux.die.net/man/2/shmctl
	//shmctl(shmid, IPC_RMID, NULL);
	
	return 0;
}
