#include <string.h> 
#include <stdio.h>

#include "swapper.h"


/**
 * Dummy implementation that demonstrates what happens if no frame is returned.
 */
struct physical_frame_entry* not_implemented_strategy(struct physical_frame_entry* physical_frames) {
	return NULL;
}

/** 
 * Dummy implementation to demonstrate Loops over all pages, sets the R_BIT to 0 and the M_BIT to 1, returns always the 
 * first physical frame
 */
struct physical_frame_entry* stupid_swapper(struct physical_frame_entry* physical_frames) {
	int i = 0;
	// loop over all physical pages
	do {
		// get a pointer to the next physical page
		struct physical_frame_entry* current_frame = physical_frames + i;
		// get the assigned virtual page for the physical page
		// (we can assume that this method is only called if all physical pages are assigned, so no NULL pointer is expected
		struct virtual_page_entry* virtual_page = current_frame->virtual_page;

		// Use makros to clear a bit
		CLEAR_BIT(virtual_page->bits, R_BIT);
		// use makro to set a bit
		SET_BIT(virtual_page->bits, M_BIT);
		i++;	
	} while (i != NUMBER_PHYSICAL_FRAMES);
	return physical_frames;
}


// global variable for the extended clock algorithm
int extended_clock_pointer = 0;
/**
 * Identifies a physical frame that can be cleared and its content 
 * swapped to disk. This method gets a pointer to an array of all 
 * our physical frames (struct physical_frame_entry is defined in 
 * swapper.h). 
 * We know that there are NUMBER_PHYSICAL_FRAMES physical frames
 * in our system.
 *
 * This method implements the extended clock algorithm to identify 
 * a proper frame to be swapped
 */
struct physical_frame_entry* extended_clock_swap_strategy(struct physical_frame_entry* physical_frames) {
	// first pass: Try to identify the first page that does neither have R- nor M-BIT set
	int moving_pointer = extended_clock_pointer; // have a local moving pointer that starts with our current_clock_pointer
	do {
		struct physical_frame_entry* current_frame = physical_frames + (moving_pointer % NUMBER_PHYSICAL_FRAMES);
		struct virtual_page_entry* virtual_page = current_frame->virtual_page;
		if (!IS_BIT_SET(virtual_page->bits, R_BIT) && !IS_BIT_SET(virtual_page->bits, M_BIT)) {
			// we have found our frame. move the pointer to that location and return the page
			extended_clock_pointer = moving_pointer;
			return current_frame;
		}
		moving_pointer = (moving_pointer + 1) % NUMBER_PHYSICAL_FRAMES;
	} while (moving_pointer != extended_clock_pointer); // move until we are back to our extended_clock_pointer

	// second pass: Try to identify a page with R_BIT not set and M_BIT SET. Clear R_BITS in the mean time
	do {
		struct physical_frame_entry* current_frame = physical_frames + (moving_pointer % NUMBER_PHYSICAL_FRAMES);
		struct virtual_page_entry* virtual_page = current_frame->virtual_page;
		if (!IS_BIT_SET(virtual_page->bits, R_BIT) && IS_BIT_SET(virtual_page->bits, M_BIT)) {
			// we have found our frame
			extended_clock_pointer = moving_pointer;
			return current_frame;
		} else {
			CLEAR_BIT(virtual_page->bits, R_BIT);
		}
		moving_pointer = (moving_pointer + 1) % NUMBER_PHYSICAL_FRAMES;
	} while (moving_pointer != extended_clock_pointer); // move until we are back to our extended_clock_pointer

	// unsuccessful at the moment, but at this point we can go again and identify a 
	return extended_clock_swap_strategy(physical_frames);
}


